#pragma once

#include <fstream>
#include <vector>
#include <algorithm>
#include <iostream>
#include <set>

#include "GeometryStructures/Node.h"
#include "GeometryStructures/FiniteElement.h"
#include "GeometryStructures/BoundaryFiniteElement.h"

class MeshLoader {

 public:
  virtual void LoadMesh(const std::string &) = 0;
  virtual ~MeshLoader() = default;

  void PrintMesh();

  std::vector<Node> &GetNodes();
  std::vector<FiniteElement> &GetFiniteElements();
  std::vector<BoundaryFiniteElement> &GetBoundaryFiniteElements();

  std::vector<FiniteElement> GetFiniteElementsByNodesId(size_t id1, size_t id2, size_t id3);
  std::vector<FiniteElement> GetFiniteElementsByEdge(size_t firstVerticeId, size_t secondVerticeId);
  std::vector<Node> GetVertexNodesBySurfaceId(size_t surfaceId);

  std::vector<FiniteElement> GetFiniteElementsByMaterialId(size_t materialId);
  std::vector<BoundaryFiniteElement> GetBboundaryFiniteElementsBySurfaceId(int id);

  std::vector<std::set<int>> GetNodesNeighbors();
 protected:
  std::vector<Node> nodes;
  std::vector<FiniteElement> finiteElements;
  std::vector<BoundaryFiniteElement> boundaryFiniteElements;
 private:
};
