#include "MeshLoaders/MeshLoader.h"

void MeshLoader::PrintMesh() {

}

std::vector<Node> &MeshLoader::GetNodes() {
  return nodes;
}

std::vector<FiniteElement> &MeshLoader::GetFiniteElements() {
  return finiteElements;
}

std::vector<BoundaryFiniteElement> &MeshLoader::GetBoundaryFiniteElements() {
  return boundaryFiniteElements;
}

std::vector<FiniteElement> MeshLoader::GetFiniteElementsByNodesId(size_t id1, size_t id2, size_t id3) {
  std::vector<FiniteElement> result;
  auto resultIter = finiteElements.begin();
  auto resultEndIter = finiteElements.end();
  while (resultIter != resultEndIter) {
    resultIter = std::find_if(resultIter, resultEndIter, [id1, id2, id3](FiniteElement currentElement) {
      auto iterBegin = currentElement.nodesId.begin();
      auto iterEnd = currentElement.nodesId.end();
      return std::find(iterBegin, iterEnd, id1) != iterEnd &&
          std::find(iterBegin, iterEnd, id2) != iterEnd &&
          std::find(iterBegin, iterEnd, id3) != iterEnd;
    });

    if (resultIter != resultEndIter) {
      result.push_back(*resultIter);
      ++resultIter;
    }
  }

  return result;
}

std::vector<FiniteElement> MeshLoader::GetFiniteElementsByEdge(size_t firstVerticeId, size_t secondVerticeId) {
  std::vector<FiniteElement> result;
  auto resultIter = finiteElements.begin();
  auto resultEndIter = finiteElements.end();
  while (resultIter != resultEndIter) {
    resultIter =
        std::find_if(resultIter, resultEndIter, [firstVerticeId, secondVerticeId](FiniteElement currentElement) {
          auto iterBegin = currentElement.nodesId.begin();
          auto iterEnd = currentElement.nodesId.end();
          return std::find(iterBegin, iterEnd, secondVerticeId) != iterEnd &&
              std::find(iterBegin, iterEnd, firstVerticeId) != iterEnd;
        });

    if (resultIter != resultEndIter) {
      result.push_back(*resultIter);
      ++resultIter;
    }
  }

  return result;
}

std::vector<Node> MeshLoader::GetVertexNodesBySurfaceId(size_t surfaceId) {
  std::vector<Node> result;
  auto currentBoundaryElement = boundaryFiniteElements.begin();
  while (currentBoundaryElement != boundaryFiniteElements.end()) {
    currentBoundaryElement = std::find_if(currentBoundaryElement,
                                          boundaryFiniteElements.end(),
                                          [surfaceId](const BoundaryFiniteElement &elem) {
                                            return elem.surfaceId == surfaceId;
                                          });
    if (currentBoundaryElement != boundaryFiniteElements.end()) {
      for (const auto &elem: currentBoundaryElement->nodesId) {
        Node nodeById = nodes[elem];
        result.push_back(nodeById);
      }
      ++currentBoundaryElement;
    }
  }
  return result;

}
std::vector<FiniteElement> MeshLoader::GetFiniteElementsByMaterialId(size_t materialId) {
  std::vector<FiniteElement> result;
  std::for_each(finiteElements.begin(), finiteElements.end(), [&materialId, &result](const FiniteElement &elem) {
    if (elem.materialId == materialId) {
      result.push_back(elem);
    }
  });
  return result;
}

std::vector<BoundaryFiniteElement> MeshLoader::GetBboundaryFiniteElementsBySurfaceId(int id) {
  std::vector<BoundaryFiniteElement> result;
  std::for_each(boundaryFiniteElements.begin(),
                boundaryFiniteElements.end(),
                [&id, &result](const BoundaryFiniteElement &elem) {
                  if (elem.surfaceId == id) {
                    result.push_back(elem);
                  }
                });
  return result;
}
std::vector<std::set<int>> MeshLoader::GetNodesNeighbors() {
  std::vector<std::set<int>> neighbors(nodes.size() + 1);
  for (const auto &elem: boundaryFiniteElements) {
    for (auto nodeId: elem.nodesId) {
      for (auto anthNodeId: elem.nodesId) {
        if (nodeId != anthNodeId) {
          neighbors[nodeId].insert(anthNodeId);
        }
      }
    }
  }

  return neighbors;
}

